;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;*
;* Number		:AVR000
;* File Name	:"2313def.inc"
;* Title		:Register/Bit Definitions for the AT90S2313
;* Date	 	:04.12.15  Dec 15, 2004
;* Version		:1.20
;* Support telephone	:+47 72 88 87 20 (ATMEL Norway)
;* Support fax	:+47 72 88 87 18 (ATMEL Norway)
;* Support E-Mail	:avr@atmel.com
;* Target MCU	:AT90S2313
;*
;* DESCRIPTION
;* When including this file in the assembly program file, all I/O register
;* names and I/O register bit names appearing in the data book can be used.
;*
;* The Register names are represented by their hexadecimal addresses.
;*
;* The Register Bit names are represented by their bit number (0-7).
;*
;* Please observe the difference in using the bit names with instructions
;* such as "sbr"/"cbr" (set/clear bit in register) and "sbrs"/"sbrc"
;* (skip if bit in register set/cleared). The following example illustrates
;* this:
;*
;* in	r16,PORTB		;read PORTB latch
;* sbr	r16,(1<<PB6)+(1<<PB5)	;set PB6 and PB5 (use masks, not bit#)
;*  use cut & paste from the lines included in the bit definitions
;* out  PORTB,r16		;output to PORTB
;*
;* in	r16,TIFR		;read the Timer Interrupt Flag Register
;* sbrc	r16,TOV0		;test the overflow flag (use bit#)
;* rjmp	TOV0_is_set		;jump if set
;* ...				;otherwise do something else
;***************************************************************************

;***** Specify Device
.device AT90S2313

;***** I/O Register Definitions
.equ	SREG	=$3f ; status register
.equ	SPL	=$3d ; stack pointer
.equ	GIMSK	=$3b ; General interrupt mask register  GIMSK $3b
.equ	GIFR	=$3a ; General interrupt flag register    GIFR $3a
.equ	TIMSK	=$39 ; Timer interrupt mask register  TIMSK $39
.equ	TIFR	=$38 ; Timer interrupt flag register    TIFR $38
.equ	MCUCR	=$35 ; microcontroller control register
.equ	MCUSR	=$34 ; microcontroller status register  MCUSR $34
.equ	TCCR0	=$33 ; timer/counter 0 control register
.equ	TCNT0	=$32 ; timer/counter 0 current count
.equ	TCCR1A	=$2f ; timer/counter control register 1A
.equ	TCCR1B	=$2e ; timer/counter control register 1B
.equ	TCNT1H	=$2d ; timer/counter 1 current count high
.equ	TCNT1L	=$2c ; timer/counter 1 current count low
.equ	OCR1AH	=$2b
.equ	OCR1AL	=$2a
.equ	ICR1H	=$25
.equ	ICR1L	=$24
.equ	WDTCR	=$21  ; watchdog timer control register
;  registers above can not use CBI or SBI instructions

.equ	EEAR	=$1e ; EEPROM address register
.equ	EEARL	=$1e
.equ	EEDR	=$1d ; EEPROM data reg
.equ	EECR	=$1c ; EEPROM control reg
.equ	PORTB	=$18 ; Port B
.equ	DDRB	=$17 ; data direction register port B
.equ	PINB	=$16 ; Port B input current logic values
.equ	PORTD	=$12
.equ	DDRD	=$11 ; data direction register port D
.equ	PIND	=$10 ; Port D input current logic values
.equ	UDR	=$0c ; UART data register
.equ	USR	=$0b ; UART status register
.equ	UCR	=$0a ; UART control register
.equ	UBRR	=$09 ; UART baud rate register
.equ	ACSR	=$08 ; analog compare


;***** Bit Definitions
;SREG status register $3f
.equ	SP7	=7 ; I  Global Interrupt Enable
.equ	SP6	=6 ; T  Bit Copy Storage
.equ	SP5	=5 ; H  Half Carry Flag
.equ	SP4	=4 ; S  Sign Flag (S = N XOR V)
.equ	SP3	=3 ; V  2's Complement Flag
.equ	SP2	=2 ; N  Negative Flag  (bit 7 set)
.equ	SP1	=1 ; Z  Zero Flag  (set when result of op is 0)
.equ	SP0	=0 ; C  Carry Flag

; GIMSK  General Interrupt Mask Register
; (0<<INT1) | (0<<INT0) ;    GIMSK $3B
.equ	INT1	=7 ; external interrupt 1 enable
.equ	INT0	=6 ; external interrupt 0 enable

; GIFR  General Interrupt Flag Register
; (0<<INTF1) | (0<<INTF0) ;    GIFR $3A
.equ	INTF1	=7 ; External interrupt 1 flag
.equ	INTF0	=6 ; External interrupt 0 flag

; TIMSK - Timer Interrupt Mask Register
; (0<<TOIE1)| (0<<OCIE1A)| (0<<TICIE)| (0<<TOIE0) ; TIMSK $39
.equ	TOIE1	=7 ; Timer 1 Overflow interrupt enable
.equ	OCIE1A	=6 ; Output compare interrupt enable
.equ	TICIE	=3 ; input capture interrupt enable
.equ	TOIE0	=1 ; Timer 0 Overflow interrupt enable

; TIFR  - Timer Flag Register
; (0<<TOV1)| (0<<OCF1A)| (0<<ICF1)| (0<<TOV0) ; TIFR $38
.equ	TOV1	=7 ; timer 1 Overflow flag
.equ	OCF1A	=6 ; Output compare flag timer 1
.equ	ICF1	=3 ; input capture flag timer 1
.equ	TOV0	=1 ; timer 0 Overflow flag

; MCUCR -  Microcontroller Control Register
; (0<<SE) | (0<<SM) | (0<<ISC11)| | (0<<ISC10) | (0<<ISC01)| (0<<ISC00) ; MCUCR $35
.equ	SE	=5 ; sleep enable
.equ	SM	=4 ; sleep mode
.equ	ISC11	=3 ; Ext Int 1 sense control 1
.equ	ISC10	=2 ; Ext Int 1 sense control 0
.equ	ISC01	=1 ; Ext Int 0 sense control bit 1
.equ	ISC00	=0 ; Ext Int 0 sense control bit 0

; timer 0 control register  TCCR0
; (0<<CS02) | (0<<CS01) | (0<<CS00) ; TCCR0 $33
.equ	CS02	=2
.equ	CS01	=1
.equ	CS00	=0

; timer 1 control register  TCCR1A
; (0<<COM1A1) | (0<<COM1A0) | (0<<PWM11) | (0<<PWM10) ; TCCR1A $2F
.equ	COM1A1	=7
.equ	COM1A0	=6
.equ	PWM11	=1
.equ	PWM10	=0

; timer 1 control register b  TCCR1B
; (0<<ICNC1) | (0<<ICES1) | (0<<CTC1) | (0<<CS12) | (0<<CS11) | (0<<CS10)  ; TCCR1B $2E
.equ	ICNC1	=7 ; input capture noise canceller 4 clocks
.equ	ICES1	=6 ; input capture edge select 1
.equ	CTC1	=3 ; clear timer/counter on compare match
.equ	CS12	=2
.equ	CS11	=1
.equ	CS10	=0

; watchdog timer control register WDTCR
; (0<<WDTOE) | (0<<WDE) | (0<<WDP2) | (0<<WDP1) | (0<<WDP0)
.equ	WDTOE	=4  ; Watch Dog Turn Off Enable
.equ	WDE	=3  ; Watch Dog Enable
.equ	WDP2	=2  ; Watch Dog Prescaler Bits  page 36
.equ	WDP1	=1
.equ	WDP0	=0

; EEPROM Address Register $1e
; (0<<EEAR6) | (0<<EEAR5) | (0<<EEAR4) | (0<<EEAR3) | (0<<EEAR2) | (0<<EEAR1) | (0<<EEAR0)

; EEPROM control register  EECR $1c
; (0<<EEMWE) | (0<<EEWE) | (0<<EERE) ; EECR $1c
.equ	EEMWE	=2  ; EEPROM Master Write Enable
.equ	EEWE	=1
.equ	EERE	=0

; Port B
; (0<<PB7) | (0<<PB6) | (0<<PB5) | (0<<PB4) | (0<<PB3) | (0<<PB2) | (0<<PB1) | (0<<PB0)
.equ	PB7	=7
.equ	PB6	=6
.equ	PB5	=5
.equ	PB4	=4
.equ	PB3	=3
.equ	PB2	=2
.equ	PB1	=1
.equ	PB0	=0

; Port B Data Direction Register
; (0<<DDB7) | (0<<DDB6) | (0<<DDB5) | (0<<DDB4) | (0<<DDB3) | (0<<DDB2) | (0<<DDB1) | (0<<DDB0)
.equ	DDB7	=7
.equ	DDB6	=6
.equ	DDB5	=5
.equ	DDB4	=4
.equ	DDB3	=3
.equ	DDB2	=2
.equ	DDB1	=1
.equ	DDB0	=0

; Port B read input register
.equ	PINB7	=7
.equ	PINB6	=6
.equ	PINB5	=5
.equ	PINB4	=4
.equ	PINB3	=3
.equ	PINB2	=2
.equ	PINB1	=1
.equ	PINB0	=0

; Port D
; (0<<PD6) | (0<<PD5) | (0<<PD4) | (0<<PD3) | (0<<PD2) | (0<<PD1) | (0<<PD0)
.equ	PD6	=6
.equ	PD5	=5
.equ	PD4	=4
.equ	PD3	=3
.equ	PD2	=2
.equ	PD1	=1
.equ	PD0	=0

; (0<<DDD6) | (0<<DDD5) | (0<<DDD4) | (0<<DDD3) | (0<<DDD2) | (0<<DDD1) | (0<<DDD0)
.equ	DDD6	=6
.equ	DDD5	=5
.equ	DDD4	=4
.equ	DDD3	=3
.equ	DDD2	=2
.equ	DDD1	=1
.equ	DDD0	=0

.equ	PIND6	=6
.equ	PIND5	=5
.equ	PIND4	=4
.equ	PIND3	=3
.equ	PIND2	=2
.equ	PIND1	=1
.equ	PIND0	=0

; UART status register  USR
.equ	RXC	=7   ; UART receive completed
.equ	TXC	=6   ; UART transmit completed
.equ	UDRE	=5   ; UART data reg empty
.equ	FE	=4   ; framing error
.equ	OVRN	=3   ; overrun error

; UART control register  UCR $0a
; (0<<RXCIE) | (0<<TXCIE) | (0<<UDRIE) | (0<<RXEN) | (0<<TXEN) | (0<<CHR9) | (0<<RXB8) | (0<<TXB8)
.equ	RXCIE	=7 ; Receive Complete int enable
.equ	TXCIE	=6 ; transmit complete int enable
.equ	UDRIE	=5 ; data reg empty int enable
.equ	RXEN	=4 ; Uart enable for receive
.equ	TXEN	=3 ; Uart enable for transmit
.equ	CHR9	=2 ; ninth bit
.equ	RXB8	=1
.equ	TXB8	=0

; (0<<ACD) | (0<<ACO) | (0<<ACI) | (0<<ACIE) | (0<<ACIC) | (0<<ACIS1) | (0<<ACIS0)
.equ	ACD	=7 ; analog compare disable
.equ	ACO	=5 ; analog compare output
.equ	ACI	=4 ; analog compare interrupt flag
.equ	ACIE	=3 ; analog compare interrupt enable
.equ	ACIC	=2 ; analog compare input compare enable
.equ	ACIS1	=1 ; Analog Compare interrupt mode select 1
.equ	ACIS0	=0 ; Analog Compare interrupt mode select 0

.def	XL	=r26
.def	XH	=r27
.def	YL	=r28
.def	YH	=r29
.def	ZL	=r30
.def	ZH	=r31

.equ    RAMEND  =$df    ;Last On-Chip SRAM Location
.equ    RAMSTART  =$60    ;Last On-Chip SRAM Location

.equ	INT0addr=$001	;External Interrupt0 Vector Address
.equ	INT1addr=$002	;External Interrupt1 Vector Address
.equ	ICP1addr=$003	;Input Capture1 Interrupt Vector Address
.equ	OC1addr =$004	;Output Compare1 Interrupt Vector Address
.equ	OVF1addr=$005	;Overflow1 Interrupt Vector Address
.equ	OVF0addr=$006	;Overflow0 Interrupt Vector Address
.equ	URXCaddr=$007	;UART Receive Complete Interrupt Vector Address
.equ	UDREaddr=$008	;UART Data Register Empty Interrupt Vector Address
.equ	UTXCaddr=$009	;UART Transmit Complete Interrupt Vector Address
.equ	ACIaddr =$00a	;Analog Comparator Interrupt Vector Address

; inline code cut-and-paste to set up AT90S2313 interrupt table
;.org $0000        ; interrupt vector table for AVR 2313
;	rjmp	reset	; Reset handler   $000
;.org INT0addr
;	reti		; External interrupt 0 handler  $001
;.org INT1addr
;	reti		; External interrupt 1 handler  $002
;.org ICP1addr
;	reti		; Timer1 capture event handler  $003
;.org OC1addr
;	reti		; Timer1 compare match     $004
;.org OVF1addr
;	reti		; Timer1 overflow handler  $005
;.org OVF0addr
;	reti		; Timer0 overflow handler  $006
;.org URXCaddr
;	reti		; UART Rx Complete $007
;.org UDREaddr
;	reti		; UART Data Register empty $008
;.org UTXCaddr
;	reti		; UART Tx Complete $009
;.org ACIaddr
;	reti		; Analog comparator handler $00a

;	            Atmel AVR AT90S2313
;	  RESET\   |1    20| VCC +5V  
;	  rxd   pd0|2    19|pb7
;	  txd   pd1|3    18|pb6
;	  XTAL1    |4    17|pb5
;	  XTAL2    |5    16|pb4
;   	int0	pd2|6    15|pb3
;  	int1	pd3|7    14|pb2
;   		pd4|8    13|pb1
;		pd5|9    12|pb0
;     	     GND   |10   11|pd6